<?xml version="1.0"?>

<!-- HDF-XSL: xsl to generate pretty HTML from HDF5 dumped to XML  -->
<!-- Version 0.1.0                                                 -->
<!-- Copyright (C) 2003-2005 David Dumas <daviddumas@gmail.com>    -->
<!-- Distributed under the terms of the GNU GPL v2.0               -->
<!-- See the file 'COPYING' for details on this license.           -->


<!-- Comments with CONFIG indicate variables with no reasonable    -->
<!-- default values.  Set these if you expect this stylesheet to   -->
<!-- generate working HTML!                                        -->

<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:hdf5="http://hdf.ncsa.uiuc.edu/DTDs/HDF5-File"
  exclude-result-prefixes="hdf5">

  <xsl:output method="html"
    encoding="utf-8"
    indent="yes"
    version="1.0"
    omit-xml-declaration="yes"
    media-type="text/html"
    standalone="yes"
    />

  <!-- I use this to include a standard "navigation" line at the top; create -->
  <!-- "nav.xsl" and uncomment to enable.                                    -->
  <!-- <xsl:include href="nav.xsl"/> -->

  <xsl:param name="filename">HDF5 File</xsl:param>
  <xsl:param name="filesize"/>  <!-- size of the HDF5 file; include on command line -->

  <!-- set with-images to nonzero value on command-line and this XSL will generate  -->
  <!-- links to images for all 2-dimensional floating-point datasets.  These images -->
  <!-- need to be generated separately.  (I use an Apache Ant task for this.)       -->
  <xsl:param name="with-images" select="0"/>  

  <!-- BASE URL FOR ALL LINKS IN THE GENERATED HTML DOCUMENT -->
  <!-- <xsl:variable name="base" select="'/~ddumas'"/> -->
  <xsl:variable name="base" select="'/'"/>  <!-- CONFIG -->

  <xsl:param name="stylesheet" select="concat($base,'/css/hdf.css')"/> <!-- CONFIG -->
  
  <!-- mini-icons path: folder, cabinet, etc. -->
  <xsl:param name="mini-icon-path" select="concat($base,'/images/mini-icon')"/>

  <!-- map from logical names to filenames of mini-icons -->
  <!-- <xsl:param name="image-path" select="concat($base,'/bear/images')"/> -->
  <xsl:param name="image-path" select="concat($base,'/images')"/> <!-- CONFIG -->
  <xsl:param name="hdf5-file">cabinet.png</xsl:param>
  <xsl:param name="group">folder.png</xsl:param>
  <xsl:param name="dataset">file.png</xsl:param>
  <xsl:param name="unknown">unknown.png</xsl:param>

  <!-- path and filenames for tree components -->
  <xsl:param name="tree-image-path" select="concat($base,'/images/tree')"/>
  <xsl:param name="tree-corner">tree_corner.png</xsl:param>
  <xsl:param name="tree-corner-top">tree_corner_top.png</xsl:param>
  <xsl:param name="tree-bar">tree_bar.png</xsl:param>
  <xsl:param name="tree-tee">tree_tee.png</xsl:param>
  <xsl:param name="tree-tee-top">tree_tee_top.png</xsl:param>
  <xsl:param name="tree-spacer">tree_spacer.png</xsl:param>

  <!-- select the nodes to be included in the navigation tree; groups and datasets -->
  <xsl:variable name="indextree" select="//*[self::hdf5:Dataset or self::hdf5:Group]"/>

  <xsl:variable name="num_datasets" select="count(//hdf5:Dataset)"/>
  <xsl:variable name="num_datasets_2d" select="count(//hdf5:Dataset[hdf5:Dataspace/hdf5:SimpleDataspace[@Ndims = 2]])"/>
  <xsl:variable name="num_groups" select="count(//*[self::hdf5:Group[hdf5:Dataset]])"/>



  <xsl:template match="hdf5:HDF5-File">
    <html>
      <head>
        <title>HDF5 File</title>
        <link rel="stylesheet" href="{$stylesheet}" type="text/css"/>
        <!--  HACK: add my mod_include boilerplate -->
        <!--       <xsl:comment>#include virtual="/~ddumas/include/hdf_header.inc"</xsl:comment> -->
      </head>
      <body>
        <!-- UNCOMMENT THE NEXT LINE IF YOU INCLUDED "nav.xsl" ABOVE -->
        <!--        <xsl:call-template name="nav"/> -->
        <h2>Contents of HDF5 file &quot;<xsl:value-of select="string($filename)"/>&quot;</h2>
        <div class="intro">
          This file contains
          <xsl:value-of select="$num_datasets"/>
          datasets in 
          <xsl:value-of select="$num_groups"/>
          leaf groups.
          <xsl:if test="$num_datasets_2d &lt; $num_datasets">
            (Note:
            <xsl:value-of select="number($num_datasets - $num_datasets_2d)"/>
            of these datasets are not displayed below because they are not 2-dimensional arrays of numbers.)
          </xsl:if>
          <br/>
          <xsl:if test="$filesize">
            File size: <xsl:value-of select="$filesize"/>
          </xsl:if>
        </div>
        <div class="fileindex">
          <h3>Dataset hierarchy</h3>
          <div class="Node">
            <img class="mini-icon" src="{$mini-icon-path}/{$hdf5-file}" alt="HDF5 File"/>
            Root
          </div>
          <xsl:apply-templates select="$indextree" mode="index"/>
        </div>
        <xsl:apply-templates select="//hdf5:Dataset" mode="details"/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="*" mode="index">
    <div class="Node">
      <xsl:apply-templates select="ancestor::*" mode="spacers"/>
      <xsl:call-template name="tee"/>
      <xsl:call-template name="node-name"/>
    </div>
  </xsl:template>

  <xsl:template name="tee">
      <xsl:choose>
        <xsl:when test="(count(following-sibling::*) &gt; 0)">
          <xsl:choose>
            <xsl:when test="(count(preceding-sibling::*) &gt; 0)">
              <img src="{$tree-image-path}/{$tree-tee}" alt="|-"/>
            </xsl:when>
            <xsl:otherwise>
              <img src="{$tree-image-path}/{$tree-tee-top}" alt="|-"/>            
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>
        <xsl:otherwise>
          <xsl:choose>
            <xsl:when test="(count(preceding-sibling::*) &gt; 0)">
              <img src="{$tree-image-path}/{$tree-corner}" alt="+-"/>
            </xsl:when>
            <xsl:otherwise>
              <img src="{$tree-image-path}/{$tree-corner-top}" alt="+-"/>            
            </xsl:otherwise>
          </xsl:choose>          
        </xsl:otherwise>
      </xsl:choose>    
  </xsl:template>

  <xsl:template name="node-name">
    <xsl:choose>
      <xsl:when test="self::hdf5:Dataset">
        <a href="#{@OBJ-XID}">
          <img class="mini-icon" src="{$mini-icon-path}/{$dataset}" alt="dataset"/>
          <xsl:value-of select="@Name"/>
        </a>
      </xsl:when>
      <xsl:when test="self::hdf5:Group">
        <img class="mini-icon" src="{$mini-icon-path}/{$group}" alt="group"/>
        <xsl:value-of select="@Name"/>
      </xsl:when>
      <xsl:otherwise>
        <img class="mini-icon" src="{$mini-icon-path}/{$unknown}" alt="unknown"/>
        <xsl:value-of select="name()"/>        
      </xsl:otherwise>
   </xsl:choose>
  </xsl:template>

  <xsl:template match="*[count(following-sibling::*) &gt; 0]" mode="spacers">
    <img src="{$tree-image-path}/{$tree-bar}" alt="|"/>
  </xsl:template>

  <xsl:template match="*" mode="spacers">
    <xsl:choose>      
      <xsl:when test="position() &gt; 2">
        <img src="{$tree-image-path}/{$tree-spacer}" alt=" "/>
      </xsl:when>
      <xsl:otherwise>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="hdf5:Dataset" mode="details">
    <div class="dataset" id="{@OBJ-XID}">
      <h3>Dataset: <xsl:value-of select="@H5Path"/></h3>
      <xsl:if test="$with-images">
        <xsl:if test="hdf5:Dataspace/hdf5:SimpleDataspace[@Ndims=2] and hdf5:DataType/hdf5:AtomicType/hdf5:FloatType">
          <a href=".{@H5Path}.png">
            <img src=".{@H5Path}.thumb.png" alt=".{H5Path}.png thumbnail"/>
          </a>
        </xsl:if>
      </xsl:if>
      <table class="dataset">
        <tr>
          <td class="attrname">Space:</td>
          <td class="attrdata"><xsl:apply-templates select="./hdf5:Dataspace" mode="brief"/></td>
        </tr>
        <tr>
          <td class="attrname">Type:</td>
          <td class="attrdata"><xsl:apply-templates select="./hdf5:DataType" mode="brief"/></td>
        </tr>
    </table>
       <xsl:if test="./hdf5:Attribute">
        <table class="attributes">
          <tr>
            <th colspan="2">
              Attributes
            </th>
          </tr>
          <xsl:for-each select="./hdf5:Attribute">
            <tr>
              <xsl:choose>
                <xsl:when test=".//hdf5:ScalarDataspace">
                  <td class="attrname"><xsl:value-of select="@Name"/></td>
                  <td class="attrdata"><xsl:value-of select=".//hdf5:DataFromFile"/></td>
                </xsl:when>
                <xsl:when test=".//hdf5:SimpleDataspace[@Ndims=1]">
                  <td class="attrname"><xsl:value-of select="@Name"/></td>
                  <td class="attrdata"><xsl:value-of select=".//hdf5:DataFromFile"/></td>
                </xsl:when>
                <xsl:otherwise>
                  <td class="attrfail">(<xsl:value-of select="@Name"/>)</td>
                  <td class="attrfail">(<xsl:apply-templates select=".//hdf5:Dataspace" mode="brief"/> / <xsl:apply-templates select=".//hdf5:DataType" mode="brief"/> )</td>
                </xsl:otherwise>

              </xsl:choose>
            </tr>
          </xsl:for-each>
        </table>
      </xsl:if>
    </div>
  </xsl:template>

  <xsl:template match="hdf5:Dataspace" mode="brief">
    <xsl:choose>
      <xsl:when test="./hdf5:ScalarDataspace">
        Scalar
      </xsl:when>
      <xsl:when test="./hdf5:SimpleDataspace">
        <xsl:for-each select="./hdf5:SimpleDataspace/hdf5:Dimension">
          <xsl:value-of select="@DimSize"/>
          <xsl:if test="following-sibling::*">x</xsl:if>
        </xsl:for-each>
        array
      </xsl:when>
      <xsl:otherwise>
        (unknown dataspace)
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="hdf5:DataType" mode="brief">
    <xsl:choose>
      <xsl:when test="./hdf5:AtomicType">
        <xsl:choose>
          <xsl:when test="./hdf5:AtomicType/hdf5:IntegerType">
            <xsl:choose>
              <xsl:when test="./hdf5:AtomicType/hdf5:IntegerType/@Sign = true">
                signed
              </xsl:when>
              <xsl:otherwise>
                unsigned
              </xsl:otherwise>
            </xsl:choose>
            <xsl:value-of select="./hdf5:AtomicType/hdf5:IntegerType/@Size"/>-byte integer
          </xsl:when>

          <xsl:when test="./hdf5:AtomicType/hdf5:FloatType">
            <xsl:value-of select="./hdf5:AtomicType/hdf5:FloatType/@Size"/>-byte float
          </xsl:when>

          <xsl:when test="./hdf5:AtomicType/hdf5:StringType">
            String
          </xsl:when>

          <xsl:when test="./hdf5:AtomicType/hdf5:TimeType">
            Time
          </xsl:when>

          <xsl:when test="./hdf5:AtomicType/hdf5:BitfieldType">
            Bit-field
          </xsl:when>

          <xsl:when test="./hdf5:AtomicType/hdf5:ReferenceType">
            (HDF5 reference)
          </xsl:when>

          <xsl:when test="./hdf5:AtomicType/hdf5:EnumType">
            Enumeration (
            <xsl:for-each select=".//hdf5:EnumType/EnumElement">
              <xsl:value-of select="."/>
              <xsl:if test="following-sibling::*">
                , &#160;
              </xsl:if>
            </xsl:for-each>
            )
          </xsl:when>

          <xsl:otherwise>
            (unknown atomic type)
          </xsl:otherwise>

        </xsl:choose>
      </xsl:when>

      <xsl:when test="./hdf5:VLType">
        Variable-length Type
      </xsl:when>

      <xsl:when test="./hdf5:ArrayType">
        Array Type
      </xsl:when>

      <xsl:otherwise>
        (unknown datatype)
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
